from sys import *
from disass3.local.disassemble import *
from disass3.local.disassemble.opcode86 import *
from disassOpcodes import *
from atlasutils.smartprint import *



"""
>>> from disassemble import *
>>> o=Opcode('\x00\x00')
>>> vars(o)
{'off': 2, 'addr_size': 4, 'dest': <disassemble.Register instance at 0xb7f8832c>, 'opcodetype': 8193, 'source': <disassemble.Expression instance at 0xb7f8830c>, 'prefix': [], 'opcode': 'add', 'mode16': 0, 'aux': None, 'data': '\x00\x00', 'cpu': 12288, 'size': 0}
>>> vars(o.dest)
{'detail': 'REG_GENERAL', 'flag': 1L, 'type': 33947649, 'name': 'al', 'size': 1}
>>> vars(o.source)
{'disp': None, 'psize': 1, 'flag': 2L, 'base': <disassemble.Register instance at 0xb7f882ec>, 'type': 33816578, 'size': 0}


   
       def __init__(self, bytes):
        self.opobj = None
        self.op1type = None
        self.op2type = None
        self.op3type = None
        self.address = 0
        self.linksto = []           # list of subs, sections, program headers, relocs these operands point to
        self.linksfrom = []         # list of subs which point to me.  Must implement "getReferenceInformation(self, address)"
        self.stringdesc = [ "" ]    # list of string information about this line of ASM.  By actually connecting to the subs and callers, this section will likely only be "sub+0x20" and such
        init(bytes)
        
"""
formatops = [ 0, 'INTEL' ]

class disassOpcode(opcode):
    ### IMPLEMENTATIONS MUST OVERRIDE THESE SUBS:
    def init(self, bytes, offset, address, mode=32):
        try:
            #print >>stderr,("mode: %x. offset: %x"%(mode, offset))
            self.opobj = Opcode(bytes, offset, mode)
            #print >>stderr,opobj.printOpcode(0)
        except:
            return None
        
    def isMEMREF(self):     # Is it interesting for Memory References?   (target or destination a Memory References)
        return (self.getOp1Type() == OPERANDTYPE_MEMORY or self.getOp2Type() == OPERANDTYPE_MEMORY or self.getOp3Type == OPERANDTYPE_MEMORY)
    
    def isJMP(self):        # Is the opcode a JMP of some sort? 
        #return self.opobj.opcode[0] == 'j'
        return (self.opobj.opcodetype & INS_CALL == INS_BRANCH)
    
    def isCALL(self):       # Is the opcode a Call of some sort?
        #return self.opobj.opcode[:2] == 'ca'
        return (self.opobj.opcodetype & INS_CALL == INS_CALL)
    
    def getOp1Type(self):
        if self.opobj.source:
            return self.getOpType(self.opobj.source)
    def getOp2Type(self):
        if self.opobj.dest:
            return self.getOpType(self.opobj.dest)
    def getOp3Type(self):
        if self.opobj.aux:
            return self.getOpType(self.opobj.aux)
    
    def getOpType(self, operand):
        #return OPERANDTYPE_MEMREF
        opt = operand.type & 0x0f00
        opr = operand.flag
        #print vars(operand)
        #print "%x:%x"%(opt,opr)
        if (opt == OP_REL or opr == OP_X): return OPERANDTYPE_MEMREF
        if (opt == OP_IMM or operand.type & ADDRMETH_E): return OPERANDTYPE_IMMEDIATE
        if (opt == OP_REG): return OPERANDTYPE_REGISTER
        if (opt == OP_UNK): return OPERANDTYPE_NONE
        
    def getLocalVar(self):
        ops = []
        if self.opobj.dest:
            ops.append(self.opobj.dest)
        if self.opobj.source:
            ops.append(self.opobj.source)
        
        for i in ops:
            try:
                if (i.base.name == 'ebp'):
                    var = i.disp.value
                    return var
            except AttributeError, e:
                pass
        return None
    
    """
fffffff4(%ebp)
{'basereg': 5L, '__module__': 'disassPyDasm', 'scale': 0L, 'dispoffset': 2L, 'dispbytes': 1L, 'sectionbytes': 0L, 'immbytes': 0L, 'displacement': -12L, 'immediate': 0L, 'reg': 8L, 'indexreg': 8L, 'flags': 117710848L, 'section': 0L, 'immoffset': 0L, 'type': 1L, '__doc__': None}
>>> vars(self.op2)

%ebx
{'basereg': 8L, '__module__': 'disassPyDasm', 'scale': 0L, 'dispoffset': 0L, 'dispbytes': 0L, 'sectionbytes': 0L, 'immbytes': 0L, 'displacement': 0L, 'immediate': 0L, 'reg': 3L, 'indexreg': 8L, 'flags': 117784576L, 'section': 0L, 'immoffset': 0L, 'type': 2L, '__doc__': None}

The first one is FFFF4(%ebp) and the second one is %ebx.
Apparently the "basereg" is used for dereffed registers and "reg" is used for direct register access
How does RegisterType work?
    """
    
    def getOpAddress(self, op):
        """ getOpAddress() is Operand-agnostic, and called by getOp1Address, getOp2Address, and getOp3Address.
        It returns an array of the following format.  If not all fields are necessary, the array is smaller:  [address, specialdescriptor(*$), segment/section]
        """
        #try:
        #print op
        #if isinstance(op, Expression) and not self.opobj.base and isinstnace(op.disp, Address):
        if isinstance(op, Expression) and isinstance(op.disp, Address):
            op = op.disp
        if isinstance(op, Address):
            ret = op.value
            if op.relative: 
                ret += self.address + len(self)
            return [ ret ]
            
        #if op:
            #opat = op.type & ADDRMETH_MASK
            #if opat == ADDRMETH_J:
                #pass
            #elif opat == ADDRMETH_I:
                #pass
            #elif opat == ADDRMETH_A:
                #pass
            
                    
        #except AttributeException,e:
            #pass
        return [ None ]
            
        #### Leftover from pydasm
        if (op.type & ADDRMETH_J):
            AM = op.flags & 0x00ff0000                              # MASK_AM per libdasm.h
            if (AM == 0x00070000):                                  # AM_J
                return [op.immediate + self.address + self.opobj.length]

            elif (AM == 0x00060000 or AM == 0x00200000):            # AM_I or AM_I
                return [op.immediate, "$"]
            
            elif (AM == 0x00010000):                                # AM_A
                return [op.displacement, "", op.section]
            
        elif (op.type == OPERAND_TYPE_MEMORY):                    ### Offset from register
            if (op.displacement > 0):
                if (op.flags & 0x0000f000 == 0x00001000):           # Execute
                    return [op.displacement, "*"]
                else:
                    return [op.displacement]
        
        ###elif (oper.type == OPERAND_TYPE_REGISTER):### WHO THE FREAK CARES?!
        return None
    
    def getOp1Address(self):
        #print >>stderr,self.getOpAddress(self.opobj.source)
        return self.getOpAddress(self.opobj.source)
    
    def getOp2Address(self):
        #print >>stderr,self.getOpAddress(self.opobj.dest)
        return self.getOpAddress(self.opobj.dest)
    
    def getOp3Address(self):
        #print >>stderr,self.getOpAddress(self.opobj.aux)
        return self.getOpAddress(self.opobj.aux)
        
    
    def toString(self):
        output = "%8x:    "%self.address
        offset = self.opobj.off - self.opobj.dataoffset
        try:
            #output += "%-35s\t%-40s\t"%(hexText(self.opobj.data[offset:self.opobj.off]),self.opobj.printOpcode(formatops[self.asmstyle], self.address))
            output += "%35s\t%-40s\t"%(hexText(self.opobj.data[:offset]),self.opobj.printOpcode(formatops[self.asmstyle], self.address))
        except:
            x,y,z = exc_info()
            excepthook(x,y,z)
            output = "%-35s"% "Error"
        
        return output
        
        
    def getTargetDistance(self, target):
        return (target - self.address)
    
    def getOpName(self):
        return self.opobj.opcode
    
    def __len__(self):
        return int(self.opobj.getSize())

"""
    Create a new opcode object
>>> get_instruction(buffer[offset:], MODE_32)

    Get String Repr of an opcode object
>>> get_instruction_string(i, FORMAT_ATT, loc)

    Get string of opcode type
>>> get_mnemonic_string(o, 0)
'add'

Address:
 |  getAddrMeth(self)
 |
 |  getFlag(self)
 |
 |  getOpSize(self)
 |
 |  getSFlag(self)
 |
 |  getSize(self)
 |
 |  getType(self)

"""
