"""
Cobra integration for remote debugging
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import md5
import os
import socket

import vtrace
import cobra

def getTracerFactory():
    """
    Return a TracerFactory proxy object from the remote server
    """
    return cobra.CobraProxy("cobra://%s:%d/TracerFactory" % (vtrace.remote, vtrace.port))

class TraceProxyFactory:
    """
    A "factory" object for creating tracers and
    wrapping them up in a proxy instance to the
    *local* server.  This object is shared out
    via the pyro server for vtrace clients.
    """
    def __init__(self, cobrad):
        self.cobrad = cobrad
        self.cobrad.shareObject(self, "TracerFactory")

    def getTrace(self):
        trace = vtrace.getTrace()
        host,port = cobra.getLocalInfo()
        unique = md5.md5(os.urandom(20)).hexdigest()
        self.cobrad.shareObject(trace, unique)
        trace.proxy = cobra.CobraProxy("cobra://%s:%d/%s" % (host,port,unique))
        return unique

    def releaseTrace(self, proxy):
        """
        When a remote system is done with a trace
        and wants the server to clean him up, hand
        the proxy object to this.
        """
        self.cobrad.unshareObject(proxy.__dict__.get("__cobra_name", None))

def getRemoteTrace():
    factory = getTracerFactory()
    unique = factory.getTrace()
    return cobra.CobraProxy("cobra://%s:%d/%s" % (vtrace.remote, vtrace.port, unique))

def startVtraceServer():
    """
    Fire up the pyro server and share out our
    "trace factory"
    """
    vtrace.cobra_daemon = cobra.CobraDaemon(port=vtrace.port)
    factory = TraceProxyFactory(vtrace.cobra_daemon)
    vtrace.cobra_daemon.fireThread()
