""" 
pydasm...  (later supplanted with libdisassemble)
 (this from help(x), where x = get_instruction("\x00\x00", pydasm.MODE_32)
class Instruction
 |  Data and other attributes defined here:
 | 
 |  dispbytes = 0L
 |  extindex = 0L
 |  flags = 0L
 |  fpuindex = 0L
 |  immbytes = 0L
 |  length = 2L
 |  mode = 0L
 |  modrm = 0L
 |  op1 = <class __main__.Operand>
 |  op2 = <class __main__.Operand>
 |  op3 = <class __main__.Operand>
 |  opcode = 0L
 |  ptr = <class __main__.Inst>
 |  sectionbytes = 0L
 |  sib = 0L
 |  type = 4L
 
>>> print pydasm.get_instruction_string(x, pydasm.FORMAT_ATT,offset)
add %al,(%eax)

"""

OPERANDTYPE_NONE = 0
OPERANDTYPE_IMMEDIATE = 1
OPERANDTYPE_REGISTER = 2
OPERANDTYPE_MEMREF = 3
ASMSTYLE_ATT = 0
ASMSTYLE_INTEL = 1
formatops = []         # this is for implementations to store their format option codes.  That way, self.formatops[self.asmstyle] can be used to abstract out the details
        
class opcode:
    """(address?,the opcode, [links], string descriptor)
    """
    def __init__(self, bytes, offset, address, mode=32, asmstyle = ASMSTYLE_ATT):
        self.opobj = None
        self.offset = offset
        self.asmstyle = asmstyle
        self.address = address
        self.linksto = None           # list of subs, sections, program headers, relocs these operands point to
        self.refsfrom = None          # list of tuples (sub, address) which reference me.  Must implement "getReferenceInformation(self, address)"
        self.jmpsfrom = None          # list of tuples (sub, address) which jmp to me.  Must implement "getReferenceInformation(self, address)"
        self.callsfrom = None         # list of tuples (sub, address) which call me.  Must implement "getReferenceInformation(self, address)"
        self.stringdesc = None    # list of string information about this line of ASM.  By actually connecting to the subs and callers, this section will likely only be "sub+0x20" and such
        self.init(bytes, offset, address, mode)
        
    ### IMPLEMENTATIONS MUST OVERRIDE THESE SUBS:
    def init(self, bytes, offset, address, mode):
        pass
    
    def isMEMREF(self):     # Is it interesting for Memory References?   (target or destination a Memory References)
        return False
    def isJMP(self):        # Is the opcode a JMP of some sort?   (self.opcode[0] == "j")???
        pass
    def isCALL(self):       # Is the opcode a Call of some sort?   (self.opcode[:2] == "ca")??
        pass

    def getLocalVar(self):
        # Search for Op1 of type OPERAND_REGISTER and look for accesses relative to %ebp
        # The idea is that no local variable is worth mentioning if it's not read from
        pass
    
    def getAddress(self):
        return self.address
    
    def addMemRef(self, src, sub):
        try:
            self.refsfrom.append((src, sub))
        except AttributeError, e:
            self.refsfrom = []
            self.refsfrom.append((src, sub))
        
    def addCaller(self, src, sub):
        try:
            self.callsfrom.append((src, sub))
        except AttributeError, e:
            self.callsfrom = []
            self.callsfrom.append((src, sub))
    
    def addJMPer(self, src, sub):
        try:
            self.jmpsfrom.append((src, sub))
        except AttributeError, e:
            self.jmpsfrom = []
            self.jmpsfrom.append((src, sub))
    
    def addLinksTo(self, dst, sub):
        try:
            self.linksto.append((dst, sub))
        except AttributeError, e:
            self.linksto = []
            self.linksto.append((dst, sub))
    
    def addStringRef(self, string):
        try:
            self.stringdesc.append(string)
        except:
            self.stringdesc = []
            self.stringdesc.append(string)
    
    def __str__(self):
        return self.printout()
    
    def printout(self, virtualmemory = None):
        output = ""
        if (virtualmemory != None):
            if (self.refsfrom or self.callsfrom or self.jmpsfrom):
                output += "\n"
            lookup = virtualmemory.lookup(self.address, findoffset = False)
            if lookup:
                output += "\t;\t" + lookup + "\n"
            if self.refsfrom:
                for s in self.refsfrom:               ### "s" is a tuple (src, sub) where  is the difference from the base address of the owner sub
                    shooter = virtualmemory.getSub(s[1])
                    name = shooter.name
                    if shooter.contains(self.address):
                        name = "(local)"
                    output += "\tReferenced by: %s   at %x\n"%(name,s[0])  # was using getQuickyRef, but why?
            if self.callsfrom:
                for s in self.callsfrom:
                    shooter = virtualmemory.getSub(s[1])
                    name = shooter.name
                    if shooter.contains(self.address):
                        name = "(local)"
                    output += "\tCalled from: %s   at %x\n"%(name,s[0])
            if self.jmpsfrom:
                for s in self.jmpsfrom:
                    shooter = virtualmemory.getSub(s[1])
                    name = shooter.name
                    if shooter.contains(self.address):
                        name = "(local)"
                    output += "\tJMPed from: %s   at %x\n"%(name,s[0])
        
        #output += "%8x:\t%-30s%-40s\t"%(hexText(self.opobj.bytes),self.address, get_instruction_string(self.opobj, self.formatops[self.asmstyle], self.address))
        ##### THIS IS THE CALL TO THE IMPLEMENTER'S SPECIFIC OPCODE-LINE PRINTING
        output += self.toString()
        
        div = "  "
        if (self.stringdesc):
            output += ";  " + "  ".join(self.stringdesc)
        else:
            div = "; "
            
        if self.linksto and (virtualmemory != None):
            output += div
            for target in self.linksto:             # tuple (address, owningsub)
                output += "%s\t "%(self.getOpName())
                tgt = virtualmemory.getSub(target[1])
                if (tgt and not tgt.contains(self.address)):
                    output += tgt.name
                    if tgt.startaddress != target[0]:
                        output += " (+0x%x)"%(target[0] - tgt.startaddress)
                else:
                    output += "(local)"
                output += "  :%+-x:    "%(self.getTargetDistance(target[0]))
        #output += "\n   %s"%(vars(self.opobj.op1))
        #output += "\n   %s"%(vars(self.opobj.op2))
        #output += "\n   %s"%(vars(self.opobj.op3))
            
        return output


    #### MUST IMPLEMENT THE FOLLOWING SUBS
    
    def getOp1Address(self):
        return [ 0xDEADBEEF, "$", 0x47145 ]
    def getOp2Address(self):
        return [ 0xDEADBEEF, "$", 0x47145 ]
    def getOp3Address(self):
        return [ 0xDEADBEEF, "$", 0x47145 ]
    
    
    def getOp1Type(self):
        return OPERANDTYPE_NONE
    def getOp2Type(self):
        return OPERANDTYPE_NONE
    def getOp3Type(self):
        return OPERANDTYPE_NONE

    def toString(self):
        return "ABSTRACT OPCODE"
    
    def __len__(self):
        return -1
