from disass3 import *
from disass3.local.Elf import *
from atlasutils.smartprint import *

FPLOOKUPLEN = 0x20
LOOKUP_NOTFOUND = ''
DATAHEXCOEFF = 1.3  # what ratio of binary to ascii should be used to determine whether to resolve to Hex or Ascii

class PHeader32(Section):
    def getBytes(self):
        return self.EXEsection.getBytes()
    
class Parser (FileParser):
    """ FileParser is a base class for ELF And PE and MACHO parsing classes to subclass from.  These subclasses should be .py files in 
        the disass/fileformats/
        What do we need from a FileFormat parser for disass?
        * sections
        * codeSections()        (returns list of sections which are Executable)
        * binarySections()      (returns list of sections which meet certain binary:ascii  ratio...  Perhaps non-exe memory is copied somewhere?
        * lookups
        * getBytes()
        * starting address
        * header information
        * program headers?????  How does this help me?  Could this be summed up in header information?
    
    def __init__(self, binary):
        self.binary = binary
        self.LOOKUPS = {}
        self.startaddress = 0
        self.filetype = "STUPID"
        self.init()
        """
        
    def validate(self, bytes):
        if bytes[:4] == '\x7fELF':
            return True
        return False
    validate = staticmethod(validate)
    
    def init(self, binary=None):
        if (binary):
            self.binary = binary
            
        self.filetype = "ELF"
        self.lowestGOT = None
        
        if (self.binary):
            self.parser = Elf(self.binary)
            self.parser.myname = self.name
            self.startaddress = self.parser.e_entry
            self.generateSections()
            self.prefillLOOKUPS()

    def prefillLOOKUPS(self):
        self.submitLookup(self.startaddress, "ELF Start")
        # for data sections
        for sec in self.parser.getSections():
            ## First off, let's stick the sections in there :)
            if (sec.sh_addr > 0):
                self.submitLookup(sec.sh_addr, sec.name)
            
                ## Next, let's dig through the section data
                offset = 0
                # check for string data using CheckDATA.
                if (sec.sh_flags & 4 == 0 and sec.name[:4] != '.got'):    # not executable
                    # For highly ascii sections, index each string
                    strings = sec.getBytes().split('\x00')
                    for string in strings:
                        if (len(string)>0):
                            self.submitLookup((sec.sh_addr + offset),"'%s' (%s) "%(SmartOutput(string, DATAHEXCOEFF), sec.name))
                        offset += len(string) + 1  #  +1 for the NULL character we just stripped off
                        
                # For highly data sections.... do nothing for now....
        # for code sections
            # hmmmm...  When does this get populated?  During disassembly.
        
        ### other RELOCS?
        for r in self.parser.relocs:   #This is just the reference in memory for GOT... not PLT
            self.submitLookup(r.r_offset, "%s()"%(r.name))
            if (not self.lowestGOT or r.r_offset < self.lowestGOT) and r.getTypeName() == 'Create PLT entry':
                self.lowestGOT = r.r_offset
        
        for s in self.parser.symbols:   #This is just the reference in memory for GOT... not PLT
            self.submitLookup(s.st_value, "%s()"%(s.name))

        for s in self.parser.dynamic_symbols:   #This is just the reference in memory for GOT... not PLT
            self.submitLookup(s.st_value, "%s()"%(s.name))

        #print >>sys.stderr,(self.LOOKUPS)
        
        ### GOT/PLT
        self.plt = self.parser.getSection('.plt')
        gotoffset = 0
        if (self.plt):
            self.pltdump = self.plt.getBytes()
            for addr in range(0x10, len(self.pltdump), 0x10):
                gotentry = struct.unpack("L", self.pltdump[addr+2:addr+6])
                pltname = self.lookup(gotentry[0])
                if pltname == None:                                     # BEGIN HACK - This is a hack for obscure PLTs like that of sshd
                    pltname = self.lookup(self.lowestGOT + gotoffset)
                    gotoffset += 4                                      # END HACK
                    
                #print >>sys.stderr,('disassELF:prefillLOOKUPS:GOT/PLT:   %x:%x %s'%((self.plt.sh_addr+addr), (gotentry[0]), pltname))
                self.submitLookup(self.plt.sh_addr+addr, "%s  (PLT)"%pltname)
            
        ### Symbols
        for sym in self.parser.getSymbols():
            if sym.st_value > 0:
                self.submitLookup(sym.st_value, "%s"%sym.name)
        
        ### dynstuff...
        
        
    def __FPlookup(self, address, deep=True):
        """ __FPlookup is the lookup for data not preloaded into the CACHE i
        If the address is still not found, returns LOOKUP_NOTFOUND """
        # Scan through sections looking for the one which contains this address
        for sec in self.getSections():
            if (address >= sec.startaddress and address <= sec.endaddress):
                offset = address - sec.startaddress
                return "(%s+0x%x: %s"%(sec.name, offset, sec.getBytes()[offset:FPLOOKUPLEN + offset])
        return ""
        
    def getSubName(self, address, section = '.text'):
        name = self.lookup(address)
        if name:
            return "%s (%x)"%(name,address)
        return "_sub_%x"%address
        
    def generateSections(self):
        """ generateSections() creates a list of disass.Section based on the ELF Sections.  This function should be called once, upon initialization.  Thereafter, getSections() should be used to get the list. """
        self.secs = []
        self.phdrs = []
        for sec in self.parser.getSections():
            newsec = ELFsection()
            newsec.EXEsection = sec     #  assuming this wants a Elf32Section....
            newsec.offset = sec.sh_offset
            newsec.startaddress = sec.sh_addr
            newsec.endaddress = sec.sh_addr + sec.sh_size
            newsec.name = sec.getName()
            newsec.offset = sec.sh_offset
            newsec.fsize = sec.sh_size
            newsec.msize = sec.sh_entsize
            if (sec.sh_flags & 4 == 0):
                newsec.type = VIRTUALMEMORYTYPE_DATA_SECTION
            else:
                newsec.type = VIRTUALMEMORYTYPE_CODE_SECTION
                if newsec.endaddress > self.endaddress:
                    self.endaddress = newsec.endaddress
            self.secs.append(newsec)
            
            
        for phdr in self.parser.getPheaders():
            newphdr = PHeader32()
            newphdr.EXEsection = phdr
            newphdr.offset = phdr.p_offset
            newphdr.startaddress = phdr.p_vaddr
            newphdr.endaddress = phdr.p_vaddr + phdr.p_memsz
            newphdr.name = phdr.getTypeName()
            newphdr.fsize = phdr.p_memsz
            newphdr.msize = phdr.p_filesz
            
            if (phdr.p_flags & 1 == 0):
                newphdr.type = VIRTUALMEMORYTYPE_DATA_SECTION
            else:
                newphdr.type = VIRTUALMEMORYTYPE_CODE_SECTION
                if (newphdr.endaddress > self.endcode):
                    self.endcode = newphdr.endaddress

            self.phdrs.append(newphdr)
            
            
    def getSections(self):
         """ returns the list of generated disass.Sections.  generateSections() should be run before this, but if not, they are generated before returning."""
         if (not self.secs):
             self.generateSections()
         return self.secs
         
    def getCodeSections(self):
        sections = []
        for sec in self.getSections():
            if (sec.isCode() and sec.getFsize() > 0):
                sections.append(sec)
        return sections
            
    def getHeaderSummary(self):
        """ getHeaders() should return a list of output for display
        This should include the GOT table, RELOCs, PLT, IMPORTs, Directory Contents, etc... """
        # File Header info
        # Program Header info
        # GOT/PLT
        # RELOCs (not covered in GOT/PLT)
        # Dynamics (external references like glibc, libz, libpthread, etc...)
        # Section Headers

    def __len__(self):    
        return len(self.parser.data)

    def __str__(self):
        output = str(self.parser)
        output += "\nDynamics:\n"
        for d in self.parser.dynamics:
            output += "%s\n"%d
        return output

    def printData(self):
        output = ""
        lastitemaddress = self.startaddress
        keys = self.LOOKUPS.keys()
        keys.sort()
        for i in keys:
            preface=" "
            output += "%s%x: %s\n"%(preface, i, self.LOOKUPS[i])
        return output
    
    def getBytes(self):
        return self.parser.data


class ELFsection(Section):
    def getBytes(self):
        return self.EXEsection.getUncompressed()

#if (not FILE_PARSERS):
#    FILE_PARSERS = []
#FILE_PARSERS.append(exeELF)
