from disass3.local.pefile import *
from disass3 import *
from atlasutils.smartprint import *

FPLOOKUPLEN = 0x20
LOOKUP_NOTFOUND = ''
DATAHEXCOEFF = 1.3  # what ratio of binary to ascii should be used to determine whether to resolve to Hex or Ascii


class Parser (FileParser):
    """ FileParser is a base class for ELF And PE and MACHO parsing classes to subclass from.  These subclasses should be .py files in 
        the disass/fileformats/
        What do we need from a FileFormat parser for disass?
        * sections
        * codeSections()        (returns list of sections which are Executable)
        * binarySections()      (returns list of sections which meet certain binary:ascii  ratio...  Perhaps non-exe memory is copied somewhere?
        * lookups
        * getBytes()
        * starting address
        * header information
        * program headers?????  How does this help me?  Could this be summed up in header information?
    
    def __init__(self, binary):
        self.binary = binary
        self.LOOKUPS = {}
        self.startaddress = 0
        self.filetype = "STUPID"
        self.init()
        """
        
    def validate(self, bytes):
        if bytes[:2] == 'MZ':
            return True
        return False
    validate = staticmethod(validate)
    
    def init(self, binary=None):
        if (binary):
            self.binary = binary
            
        self.filetype = "PE"
        if (self.binary):
            self.parser = PE(data=self.binary)
            #self.parser.myname = self.name
            self.startaddress = self.parser.OPTIONAL_HEADER.AddressOfEntryPoint + self.parser.OPTIONAL_HEADER.ImageBase
            self.endcode = self.parser.sections[0].VirtualAddress + self.parser.OPTIONAL_HEADER.ImageBase + self.parser.sections[0].SizeOfRawData
            self.generateSections()
            self.prefillLOOKUPS()
            

    def prefillLOOKUPS(self):
        self.submitLookup(self.startaddress, "PE Start")
        # for data sections
        for sec in self.getSections():
            ## First off, let's stick the sections in there :)
            if (sec.startaddress > 0):
                self.submitLookup(sec.startaddress, sec.name)
            
            ## Next, let's dig through the section data
            offset = 0
            # check for string data using CheckDATA.
            if (sec.isData()):    # not executable
                # For highly ascii sections, index each string
                strings = sec.getBytes().split('\x00')
                for string in strings:
                    offset += len(string) + 1  #  +1 for the NULL character we just stripped off
                    if (len(string)>0):
                        self.submitLookup((sec.startaddress + offset),"'%s' "%SmartOutput(string, DATAHEXCOEFF))
                # For highly data sections.... do nothing for now....
        # for code sections
            # hmmmm...  When does this get populated?  During disassembly.
        
        ### other RELOCS?
        #for r in self.parser.getRelocs():   #This is just the reference in memory for GOT... not PLT
            ##print >>sys.stderr,("disassELF:prefillLOOKUPS:RELOCS:  %x: %s"%(r.r_offset,  "%s  (%s)"%(r.name, r.getTypeName())))
            ##self.submitLookup(r.r_offset, "%s  (%s)"%(r.name, r.getTypeName()))
            #self.submitLookup(r.r_offset, "%s()"%(r.name))
        
        """
        TREE::'DIRECTORY_ENTRY_TLS':'struct':'values':'AddressOfCallBacks' <type 'long'> == 4304932
        TREE::'DIRECTORY_ENTRY_TLS':'struct':'values':'Characteristics' <type 'long'> == 0
        TREE::'DIRECTORY_ENTRY_TLS':'struct':'values':'EndAddressOfRawData' <type 'long'> == 4415500
        TREE::'DIRECTORY_ENTRY_TLS':'struct':'values':'AddressOfIndex' <type 'long'> == 4406420
        TREE::'DIRECTORY_ENTRY_TLS':'struct':'values':'StartAddressOfRawData' <type 'long'> == 4415488
        TREE::'DIRECTORY_ENTRY_TLS':'struct':'values':'SizeOfZeroFill' <type 'long'> == 0
        
        
        TREE::'FILE_HEADER':'values':'NumberOfSections' <type 'int'> == 5
        TREE::'FILE_HEADER':'values':'TimeDateStamp' <type 'long'> == 1065045697
        TREE::'FILE_HEADER':'values':'PointerToSymbolTable' <type 'long'> == 0
        TREE::'FILE_HEADER':'values':'NumberOfSymbols' <type 'long'> == 0
        TREE::'FILE_HEADER':'values':'Machine' <type 'int'> == 332
        TREE::'FILE_HEADER':'values':'Characteristics' <type 'int'> == 271
        TREE::'FILE_HEADER':'values':'SizeOfOptionalHeader' <type 'int'> == 224

        """
            
        #print >>sys.stderr,(self.LOOKUPS)
        
        ### GOT/PLT  (windows style)
        """
        TREE::'DIRECTORY_ENTRY_IMPORT':[1]:'imports':[0]:'name' <type 'str'> == GetClipboardData
        TREE::'DIRECTORY_ENTRY_IMPORT':[1]:'imports':[0]:'address' <type 'long'> == 4289104
        """
        for dll in self.parser.DIRECTORY_ENTRY_IMPORT:
            for imprt in dll.imports:
                impname = imprt.name
                if impname == None:
                    impname = "ordinal-%d"%imprt.ordinal
                self.submitLookup(imprt.address, "%s:%s"%(dll.dll, impname))
                
            
        ### Symbols
        
        ### dynstuff...
        
        
    def __FPlookup(self, address, deep=True):
        """ __FPlookup is the lookup for data not preloaded into the CACHE i
        If the address is still not found, returns LOOKUP_NOTFOUND """
        # Scan through sections looking for the one which contains this address
        for sec in self.getSections():
            if (address >= sec.startaddress and address <= sec.endaddress):
                offset = address - sec.startaddress
                return "(%s+0x%x: %s"%(sec.name, offset, sec.getBytes()[offset:FPLOOKUPLEN + offset])
        return ""
        
    def getSubName(self, address, section = '.text'):
        name = self.lookup(address)
        if name:
            return "%s (%x)"%(name,address)
        return "_sub_%x"%address
        
    def generateSections(self):
        """ generateSections() creates a list of disass.Section based on the ELF Sections.  This function should be called once, upon initialization.  Thereafter, getSections() should be used to get the list. """
        
        """
        TREE::'sections':[0]:'values': (<type 'dict'>)
        TREE::'sections':[0]:'values':'PointerToRawData' <type 'long'> == 4096
        TREE::'sections':[0]:'values':'Name' <type 'str'> == .text
        TREE::'sections':[0]:'values':'NumberOfRelocations' <type 'int'> == 0
        TREE::'sections':[0]:'values':'Misc_VirtualSize' <type 'long'> == 89458
        TREE::'sections':[0]:'values':'Characteristics' <type 'long'> == 3758096416
        TREE::'sections':[0]:'values':'Misc' <type 'long'> == 89458
        TREE::'sections':[0]:'values':'PointerToLinenumbers' <type 'long'> == 0
        TREE::'sections':[0]:'values':'SizeOfRawData' <type 'long'> == 90112
        TREE::'sections':[0]:'values':'Misc_PhysicalAddress' <type 'long'> == 89458
        TREE::'sections':[0]:'values':'PointerToRelocations' <type 'long'> == 0
        TREE::'sections':[0]:'values':'VirtualAddress' <type 'long'> == 4096
        TREE::'sections':[0]:'values':'NumberOfLinenumbers' <type 'int'> == 0
        """
        
        self.secs = []
        
        for sec in self.parser.sections:
            newsec = PEsection()
            newsec.EXEsection = sec     #  assuming this wants a Elf32Section....
            newsec.offset = sec.PointerToRawData
            newsec.startaddress = sec.VirtualAddress + self.parser.OPTIONAL_HEADER.ImageBase
            newsec.endaddress = newsec.startaddress + sec.SizeOfRawData
            newsec.name = sec.Name
            
            newsec.fsize = sec.SizeOfRawData
            newsec.msize = sec.Misc_VirtualSize
            if (sec.Name[:5] == '.text' or sec.Characteristics & 0x20000000 ):
                newsec.type = VIRTUALMEMORYTYPE_CODE_SECTION
            else:
                newsec.type = VIRTUALMEMORYTYPE_DATA_SECTION
            self.secs.append(newsec)
            
            if newsec.endaddress > self.endaddress:
                self.endaddress = newsec.endaddress

            
            
    def getSections(self):
         """ returns the list of generated disass.Sections.  generateSections() should be run before this, but if not, they are generated before returning."""
         if (not self.secs):
             self.generateSections()
         return self.secs
         
    def getCodeSections(self):
        sections = []
        for sec in self.getSections():
            if (sec.isCode()):
                sections.append(sec)
        return sections
            
    def getHeaderSummary(self):
        """ getHeaders() should return a list of output for display
        This should include the GOT table, RELOCs, PLT, IMPORTs, Directory Contents, etc... """
        # File Header info
        # Program Header info
        # GOT/PLT
        # RELOCs (not covered in GOT/PLT)
        # Dynamics (external references like glibc, libz, libpthread, etc...)
        # Section Headers
        
    def printData(self):
        output = ""
        lastitemaddress = self.startaddress
        keys = self.LOOKUPS.keys()
        keys.sort()
        for i in keys:
            preface=" "
            output += "%s%x: %s\n"%(preface, i, self.LOOKUPS[i])
        return output

    def __len__(self):
        return len(self.parser.__data__)
    
    def __str__(self):
        return str(self.parser)
    
    def getBytes(self):
        return self.parser



class PEsection(Section):
    def getBytes(self):
        return self.EXEsection.data
#if (not FILE_PARSERS):
#    FILE_PARSERS = []
#FILE_PARSERS.append(exeELF)
