"""
Vtrace notitifers base classes and examples

Vtrace supports the idea of callback notifiers which
get called whenever particular events occur in the target
process.  Notifiers may be registered to recieve a callback
on any of the NOTIFY_FOO events from vtrace.  One notifier
*may* be registered with more than one trace, as the "notify"
method is passed a reference to the trace for which an event
has occured...

"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
from vtrace import *

class Notifier:
    """
    The top level example notifier...  Anything which registers
    itself for trace events or tracegroup events should implement
    the notify method as shown here.
    """

    def __init__(self):
        """
        All extenders *must* call this.  Mostly because all the
        goop necissary for the remote debugging stuff...
        (if notifier is instantiated on server, all is well, if it's
        on the client it needs a proxy...)
        """
        pass
        #self.proxy = None
        #if vtrace.remote: # We have a remote server!
            #self.proxy = getObjectProxy(self)

    def handleEvent(self, event, trace):
        """
        An "internal" handler so if we need to do something
        from an API perspective before calling the notify method
        we can have a good "all at once" hook
        """
        self.notify(event, trace)

    def notify(self, event, trace):
        print "Got event: %d from pid %d" % (event, trace.getPid())

class VerboseNotifier(Notifier):
    def notify(self, event, trace):
        print "PID %d thread(%d) got" % (trace.getPid(), trace.getMeta("ThreadId")),
        if event == NOTIFY_ALL:
            print "WTF, how did we get a NOTIFY_ALL event?!?!"
        elif event == NOTIFY_SIGNAL:
            print "NOTIFY_SIGNAL"
            print "PendingSignal",trace.getMeta("PendingSignal")
            print "PendingException",trace.getMeta("PendingException")
        elif event == NOTIFY_BREAK:
            print "NOTIFY_BREAK"
        elif event == NOTIFY_SYSCALL:
            print "NOTIFY_SYSCALL"
        elif event == NOTIFY_CONTINUE:
            print "NOTIFY_CONTINUE"
        elif event == NOTIFY_EXIT:
            print "NOTIFY_EXIT"
            print "ExitCode",trace.getMeta("ExitCode")
        elif event == NOTIFY_ATTACH:
            print "NOTIFY_ATTACH"
        elif event == NOTIFY_DETACH:
            print "NOTIFY_DETACH"
        elif event == NOTIFY_LOAD_LIBRARY:
            print "NOTIFY_LOAD_LIBRARY"
        elif event == NOTIFY_UNLOAD_LIBRARY:
            print "NOTIFY_UNLOAD_LIBRARY"
        elif event == NOTIFY_CREATE_THREAD:
            print "NOTIFY_CREATE_THREAD"
        elif event == NOTIFY_EXIT_THREAD:
            print "NOTIFY_EXIT_THREAD"
            print "ExitThread",trace.getMeta("ExitThread", -1)
        elif event == NOTIFY_STEP:
            print "NOTIFY_STEP"
        else:
            print "NOTIFY_WTF_HUH?"

