"""
Darwin Platform Module
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import struct
import vtrace.platforms.posix as v_posix
import vtrace.symbase as symbase

class MachoSymbolResolver(symbase.VSymbolResolver):
    pass

class MachoMixin:
    def platformGetSymbolResolver(self, filename, baseaddr):
        return MachoSymbolResolver(filename, baseaddr)

i386_thread_state = 1
i386_thread_state64 = 4
i386_debug_state = 10
i386_debug_state64 = 11

class DarwinMixin:

    def initMixin(self):
        self.tdict = {}
        self.setMeta("Platform", "Darwin")

    def platformPs(self):
        import mach
        return mach.process_list()

    def getExe(self):
        print "FIXME getExe() on darwin"
        return ""

    def platformGetFds(self):
        print "FIXME platformGetFds() no workie on darwin yet..."
        return []

    def platformGetMaps(self):
        return self.task.get_mmaps()

    def platformReadMemory(self, address, length):
        return self.task.vm_read(address, length)

    def platformWriteMemory(self, address, buffer):
        return self.task.vm_write(address, buffer)

    def currentMachThread(self):
        self.getThreads()
        return self.tdict[self.getMeta("ThreadId")]

    def platformGetRegs(self):
        """
        """
        thr = self.currentMachThread()
        regs = thr.get_state(i386_thread_state)
        return regs + thr.get_state(i386_debug_state)

    def platformSetRegs(self, regbuf):
        thr = self.currentMachThread()
        thr.set_state(1, regbuf[:-32])
        thr.set_state(10,regbuf[-32:])

    def platformGetThreads(self):
        ret = {}
        self.tdict = {}
        for thread in self.task.threads():
            # FIXME this is a platform dependant hack for now
            esp = struct.unpack("16L",thread.get_state(1)[:64])[7]
            mapbase,maplen,mperm,mfile = self.getMap(esp)
            tid = mapbase+maplen # The TOP of the stack, so it doesn't grow down and change
            ret[tid] = tid
            self.tdict[tid] = thread
        self.setMeta("ThreadId", tid) #FIXME how can we know what thread caused an event?
        return ret

    def platformAttach(self, pid):
        import mach
        self.task = mach.task_for_pid(pid)
        v_posix.PtraceMixin.platformAttach(self, pid)

class DarwinIntelRegisters:
    """
    Mixing for the register format of Darwin on Intel
    """
    def getRegisterFormat(self):
        return "24L"

    def getRegisterNames(self):
        return ("eax","ebx","ecx","edx","edi",
                "esi","ebp","esp","ss","eflags",
                "eip","cs","ds","es","fs","gs",
                "debug0","debug1","debug2","debug3",
                "debug4","debug5","debug6","debug7")

class DarwinPpcRegisters:
    def getRegisterFormat(self):
        return "40L"

    def getRegisterNames(self):
        mylist = []
        for i in range(40):
            mylist.append("r%d" % i)
        return mylist

