"""
Win32 Platform Module
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
import os
import sys
import struct
import traceback

import vtrace
import vtrace.symbase as symbase

from ctypes import *
#from ctypes.wintypes import *

platdir = os.path.dirname(__file__)

psapi = None

kernel32 = None
dbghelp = None
psapi = None

if sys.platform == "win32":
    kernel32 = windll.kernel32
    dbghelp = windll.LoadLibrary(os.path.join(platdir, "dbghelp.dll"))

    try:
        psapi = windll.psapi
    except:
        print "WARNING: psapi not found"

INFINITE = 0xffffffff
EXCEPTION_MAXIMUM_PARAMETERS = 15

# Debug Event Types
EXCEPTION_DEBUG_EVENT       =1
CREATE_THREAD_DEBUG_EVENT   =2
CREATE_PROCESS_DEBUG_EVENT  =3
EXIT_THREAD_DEBUG_EVENT     =4
EXIT_PROCESS_DEBUG_EVENT    =5
LOAD_DLL_DEBUG_EVENT        =6
UNLOAD_DLL_DEBUG_EVENT      =7
OUTPUT_DEBUG_STRING_EVENT   =8
RIP_EVENT                   =9

# Symbol Flags
SYMFLAG_VALUEPRESENT     = 0x00000001
SYMFLAG_REGISTER         = 0x00000008
SYMFLAG_REGREL           = 0x00000010
SYMFLAG_FRAMEREL         = 0x00000020
SYMFLAG_PARAMETER        = 0x00000040
SYMFLAG_LOCAL            = 0x00000080
SYMFLAG_CONSTANT         = 0x00000100
SYMFLAG_EXPORT           = 0x00000200
SYMFLAG_FORWARDER        = 0x00000400
SYMFLAG_FUNCTION         = 0x00000800
SYMFLAG_VIRTUAL          = 0x00001000
SYMFLAG_THUNK            = 0x00002000
SYMFLAG_TLSREL           = 0x00004000

# Symbol Resolution Options
SYMOPT_CASE_INSENSITIVE         = 0x00000001
SYMOPT_UNDNAME                  = 0x00000002
SYMOPT_DEFERRED_LOADS           = 0x00000004
SYMOPT_NO_CPP                   = 0x00000008
SYMOPT_LOAD_LINES               = 0x00000010
SYMOPT_OMAP_FIND_NEAREST        = 0x00000020
SYMOPT_LOAD_ANYTHING            = 0x00000040
SYMOPT_IGNORE_CVREC             = 0x00000080
SYMOPT_NO_UNQUALIFIED_LOADS     = 0x00000100
SYMOPT_FAIL_CRITICAL_ERRORS     = 0x00000200
SYMOPT_EXACT_SYMBOLS            = 0x00000400
SYMOPT_ALLOW_ABSOLUTE_SYMBOLS   = 0x00000800
SYMOPT_IGNORE_NT_SYMPATH        = 0x00001000
SYMOPT_INCLUDE_32BIT_MODULES    = 0x00002000
SYMOPT_PUBLICS_ONLY             = 0x00004000
SYMOPT_NO_PUBLICS               = 0x00008000
SYMOPT_AUTO_PUBLICS             = 0x00010000
SYMOPT_NO_IMAGE_SEARCH          = 0x00020000
SYMOPT_SECURE                   = 0x00040000
SYMOPT_NO_PROMPTS               = 0x00080000
SYMOPT_DEBUG                    = 0x80000000

# Exception Types
EXCEPTION_WAIT_0                     = 0x00000000L    
EXCEPTION_ABANDONED_WAIT_0           = 0x00000080L    
EXCEPTION_USER_APC                   = 0x000000C0L    
EXCEPTION_TIMEOUT                    = 0x00000102L    
EXCEPTION_PENDING                    = 0x00000103L    
DBG_EXCEPTION_HANDLED             = 0x00010001L    
DBG_CONTINUE                      = 0x00010002L    
EXCEPTION_SEGMENT_NOTIFICATION       = 0x40000005L    
DBG_TERMINATE_THREAD              = 0x40010003L    
DBG_TERMINATE_PROCESS             = 0x40010004L    
DBG_CONTROL_C                     = 0x40010005L    
DBG_CONTROL_BREAK                 = 0x40010008L    
DBG_COMMAND_EXCEPTION             = 0x40010009L    
EXCEPTION_GUARD_PAGE_VIOLATION       = 0x80000001L    
EXCEPTION_DATATYPE_MISALIGNMENT      = 0x80000002L    
EXCEPTION_BREAKPOINT                 = 0x80000003L    
EXCEPTION_SINGLE_STEP                = 0x80000004L    
DBG_EXCEPTION_NOT_HANDLED         = 0x80010001L    
EXCEPTION_ACCESS_VIOLATION           = 0xC0000005L    
EXCEPTION_IN_PAGE_ERROR              = 0xC0000006L    
EXCEPTION_INVALID_HANDLE             = 0xC0000008L    
EXCEPTION_NO_MEMORY                  = 0xC0000017L    
EXCEPTION_ILLEGAL_INSTRUCTION        = 0xC000001DL    
EXCEPTION_NONCONTINUABLE_EXCEPTION   = 0xC0000025L    
EXCEPTION_INVALID_DISPOSITION        = 0xC0000026L    
EXCEPTION_ARRAY_BOUNDS_EXCEEDED      = 0xC000008CL    
EXCEPTION_FLOAT_DENORMAL_OPERAND     = 0xC000008DL    
EXCEPTION_FLOAT_DIVIDE_BY_ZERO       = 0xC000008EL    
EXCEPTION_FLOAT_INEXACT_RESULT       = 0xC000008FL    
EXCEPTION_FLOAT_INVALID_OPERATION    = 0xC0000090L    
EXCEPTION_FLOAT_OVERFLOW             = 0xC0000091L    
EXCEPTION_FLOAT_STACK_CHECK          = 0xC0000092L    
EXCEPTION_FLOAT_UNDERFLOW            = 0xC0000093L    
EXCEPTION_INTEGER_DIVIDE_BY_ZERO     = 0xC0000094L    
EXCEPTION_INTEGER_OVERFLOW           = 0xC0000095L    
EXCEPTION_PRIVILEGED_INSTRUCTION     = 0xC0000096L    
EXCEPTION_STACK_OVERFLOW             = 0xC00000FDL    
EXCEPTION_CONTROL_C_EXIT             = 0xC000013AL    
EXCEPTION_FLOAT_MULTIPLE_FAULTS      = 0xC00002B4L    
EXCEPTION_FLOAT_MULTIPLE_TRAPS       = 0xC00002B5L    
EXCEPTION_REG_NAT_CONSUMPTION        = 0xC00002C9L    

# Context Info
CONTEXT_i386    = 0x00010000    # this assumes that i386 and
CONTEXT_i486    = 0x00010000    # i486 have identical context records
CONTEXT_CONTROL         = (CONTEXT_i386 | 0x00000001L) # SS:SP, CS:IP, FLAGS, BP
CONTEXT_INTEGER         = (CONTEXT_i386 | 0x00000002L) # AX, BX, CX, DX, SI, DI
CONTEXT_SEGMENTS        = (CONTEXT_i386 | 0x00000004L) # DS, ES, FS, GS
CONTEXT_FLOATING_POINT  = (CONTEXT_i386 | 0x00000008L) # 387 state
CONTEXT_DEBUG_REGISTERS = (CONTEXT_i386 | 0x00000010L) # DB 0-3,6,7
CONTEXT_EXTENDED_REGISTERS  = (CONTEXT_i386 | 0x00000020L) # cpu specific extensions
CONTEXT_FULL = (CONTEXT_CONTROL | CONTEXT_INTEGER | CONTEXT_SEGMENTS)
CONTEXT_ALL = (CONTEXT_CONTROL | CONTEXT_INTEGER | CONTEXT_SEGMENTS | CONTEXT_FLOATING_POINT | CONTEXT_DEBUG_REGISTERS | CONTEXT_EXTENDED_REGISTERS)

# Thread Permissions
THREAD_ALL_ACCESS = 0x001f03ff
PROCESS_ALL_ACCESS = 0x001f0fff

# Memory Permissions
PAGE_NOACCESS = 0x01
PAGE_READONLY = 0x02
PAGE_READWRITE = 0x04
PAGE_WRITECOPY = 0x08
PAGE_EXECUTE = 0x10
PAGE_EXECUTE_READ = 0x20
PAGE_EXECUTE_READWRITE = 0x40
PAGE_EXECUTE_WRITECOPY = 0x80
PAGE_GUARD = 0x100
PAGE_NOCACHE = 0x200
PAGE_WRITECOMBINE = 0x400

# Memory States
MEM_COMMIT = 0x1000
MEM_FREE = 0x10000
MEM_RESERVE = 0x2000

# Memory Types
MEM_IMAGE = 0x1000000
MEM_MAPPED = 0x40000
MEM_PRIVATE = 0x20000

# Process Creation Flags
DEBUG_ONLY_THIS_PROCESS = 0x02

MAX_PATH=260

class EXCEPTION_RECORD(Structure):
    _fields_ = [
            ("ExceptionCode", c_ulong),
            ("ExceptionFlags", c_ulong),
            ("ExceptionRecord", c_void_p),
            ("ExceptionAddress", c_void_p),
            ("NumberParameters", c_ulong),
            ("ExceptionInformation", c_ulong * EXCEPTION_MAXIMUM_PARAMETERS)
            ]

class EXCEPTION_DEBUG_INFO(Structure):
    _fields_ = [
            ("ExceptionRecord", EXCEPTION_RECORD),
            ("FirstChance", c_ulong)
            ]
class CREATE_THREAD_DEBUG_INFO(Structure):
    _fields_ = [
            ("Thread", c_ulong),
            ("ThreadLocalBase", c_ulong),
            ("StartAddress", c_ulong)
            ]
class CREATE_PROCESS_DEBUG_INFO(Structure):
    _fields_ = [
            ("File", c_ulong), # HANDLE 
            ("Process", c_ulong), # HANDLE
            ("Thread", c_ulong), # HANDLE
            ("BaseOfImage", c_ulong),
            ("DebugInfoFileOffset", c_ulong),
            ("DebugInfoSize", c_ulong),
            ("ThreadLocalBase", c_ulong),
            ("StartAddress", c_ulong),
            ("ImageName", c_ulong),
            ("Unicode", c_short),
            ]
class EXIT_THREAD_DEBUG_INFO(Structure):
    _fields_ = [("ExitCode", c_ulong),]
class EXIT_PROCESS_DEBUG_INFO(Structure):
    _fields_ = [("ExitCode", c_ulong),]
class LOAD_DLL_DEBUG_INFO(Structure):
    _fields_ = [
            ("File", c_ulong), #HANDLE
            ("BaseOfDll", c_ulong),
            ("DebugInfoFileOffset", c_ulong),
            ("DebugInfoSize", c_ulong),
            ("ImageName", c_ulong),
            ("Unicode", c_ushort),
            ]
class UNLOAD_DLL_DEBUG_INFO(Structure):
    _fields_ = [
            ("BaseOfDll", c_ulong),
            ]
class OUTPUT_DEBUG_STRING_INFO(Structure):
    _fields_ = [
            ("DebugStringData", c_char_p),
            ("Unicode", c_ushort),
            ("DebugStringLength", c_ushort),
            ]
class RIP_INFO(Structure):
    _fields_ = [
            ("Error", c_ulong),
            ("Type", c_ulong),
            ]

class DBG_EVENT_UNION(Union):
    _fields_ = [ ("Exception",EXCEPTION_DEBUG_INFO),
                 ("CreateThread", CREATE_THREAD_DEBUG_INFO),
                 ("CreateProcessInfo", CREATE_PROCESS_DEBUG_INFO),
                 ("ExitThread", EXIT_THREAD_DEBUG_INFO),
                 ("ExitProcess", EXIT_PROCESS_DEBUG_INFO),
                 ("LoadDll", LOAD_DLL_DEBUG_INFO),
                 ("UnloadDll", UNLOAD_DLL_DEBUG_INFO),
                 ("DebugString", OUTPUT_DEBUG_STRING_INFO),
                 ("RipInfo", RIP_INFO)]

class DEBUG_EVENT(Structure):
    _fields_ = [
            ("DebugEventCode", c_ulong),
            ("ProcessId", c_ulong),
            ("ThreadId", c_ulong),
            ("u", DBG_EVENT_UNION),
            ]

class FloatSavex86(Structure):
    _fields_ = [("ControlWord", c_ulong),
                  ("StatusWord", c_ulong),
                  ("TagWord", c_ulong),
                  ("ErrorOffset", c_ulong),
                  ("ErrorSelector", c_ulong),
                  ("DataOffset", c_ulong),
                  ("DataSelector", c_ulong),
                  ("RegisterSave", c_byte*80),
                  ("Cr0NpxState", c_ulong),
                  ]

class CONTEXTx86(Structure):
    _fields_ = [ ("ContextFlags", c_ulong),
                   ("Dr0", c_ulong),
                   ("Dr1", c_ulong),
                   ("Dr2", c_ulong),
                   ("Dr3", c_ulong),
                   ("Dr4", c_ulong),
                   ("Dr5", c_ulong),
                   ("Dr6", c_ulong),
                   ("Dr7", c_ulong),
                   ("FloatSave", FloatSavex86),
                   ("SegGs", c_ulong),
                   ("SegFs", c_ulong),
                   ("SegEs", c_ulong),
                   ("SegDs", c_ulong),
                   ("edi", c_ulong),
                   ("esi", c_ulong),
                   ("ebx", c_ulong),
                   ("edx", c_ulong),
                   ("ecx", c_ulong),
                   ("eax", c_ulong),
                   ("ebp", c_ulong),
                   ("eip", c_ulong),
                   ("SegCs", c_ulong),
                   ("eflags", c_ulong),
                   ("esp", c_ulong),
                   ("SegSs", c_ulong),
                   ("Extension", c_byte * 512),
                   ]

class MEMORY_BASIC_INFORMATION32(Structure):
    _fields_ = [
        ("BaseAddress", c_ulong),
        ("AllocationBase", c_ulong),
        ("AllocationProtect", c_ulong),
        ("RegionSize", c_ulong),
        ("State", c_ulong),
        ("Protect", c_ulong),
        ("Type", c_ulong),
        ]

class MEMORY_BASIC_INFORMATION64(Structure):
    _fields_ = [
        ("BaseAddress", c_ulonglong),
        ("AllocationBase", c_ulonglong),
        ("AllocationProtect", c_ulong),
        ("alignment1", c_ulong),
        ("RegionSize", c_ulonglong),
        ("State", c_ulong),
        ("Protect", c_ulong),
        ("Type", c_ulong),
        ("alignment2", c_ulong),
        ]

class STARTUPINFO(Structure):
    """
    Passed into CreateProcess
    """
    _fields_ = [
            ("db", c_ulong),
            ("Reserved", c_char_p),
            ("Desktop", c_char_p),
            ("Title", c_char_p),
            ("X", c_ulong),
            ("Y", c_ulong),
            ("XSize", c_ulong),
            ("YSize", c_ulong),
            ("XCountChars", c_ulong),
            ("YCountChars", c_ulong),
            ("FillAttribute", c_ulong),
            ("Flags", c_ulong),
            ("ShowWindow", c_ushort),
            ("Reserved2", c_ushort),
            ("Reserved3", c_void_p),
            ("StdInput", c_ulong),
            ("StdOutput", c_ulong),
            ("StdError", c_ulong),
            ]

class PROCESS_INFORMATION(Structure):
    _fields_ = [
            ("Process", c_ulong),
            ("Thread", c_ulong),
            ("ProcessId", c_ulong),
            ("ThreadId", c_ulong),
            ]

class SYMBOL_INFO(Structure):
    _fields_ = [
                ("SizeOfStruct", c_ulong),
                ("TypeIndex", c_ulong),
                ("Reserved1", c_ulonglong),
                ("Reserved2", c_ulonglong),
                ("Info", c_ulong),
                ("Size", c_ulong),
                ("ModBase", c_ulonglong),
                ("Flags", c_ulong),
                ("Value", c_ulonglong),
                ("Address", c_ulonglong),
                ("Register", c_ulong),
                ("Scope", c_ulong),
                ("Tag", c_ulong),
                ("NameLen", c_ulong),
                ("MaxNameLen", c_ulong),
                ("Name", c_char * 2000), # MAX_SYM_NAME
                ]

class IMAGEHLP_MODULE64(Structure):
    _fields_ = [
            ("SizeOfStruct", c_ulong),
            ("BaseOfImage", c_ulonglong),
            ("ImageSize", c_ulong),
            ("TimeDateStamp", c_ulong),
            ("CheckSum", c_ulong),
            ("NumSyms", c_ulong),
            ("SymType", c_ulong),
            ("ModuleName", c_char*32),
            ("ImageName", c_char*256),
            ("LoadedImageName", c_char*256),
            ("LoadedPdbName", c_char*256),
            ("CvSig", c_ulong),
            ("CvData", c_char*(MAX_PATH*3)),
            ("PdbSig", c_ulong),
            ("PdbSig70", c_char * 16), #GUID
            ("PdbAge", c_ulong),
            ("PdbUnmatched", c_ulong),
            ("DbgUnmatched", c_ulong),
            ("LineNumbers", c_ulong),
            ("GlobalSymbols", c_ulong),
            ("TypeInfo", c_ulong),
            ]


IMAGE_DIRECTORY_ENTRY_EXPORT          =0   # Export Directory
IMAGE_DIRECTORY_ENTRY_IMPORT          =1   # Import Directory
IMAGE_DIRECTORY_ENTRY_RESOURCE        =2   # Resource Directory
IMAGE_DIRECTORY_ENTRY_EXCEPTION       =3   # Exception Directory
IMAGE_DIRECTORY_ENTRY_SECURITY        =4   # Security Directory
IMAGE_DIRECTORY_ENTRY_BASERELOC       =5   # Base Relocation Table
IMAGE_DIRECTORY_ENTRY_DEBUG           =6   # Debug Directory
IMAGE_DIRECTORY_ENTRY_COPYRIGHT       =7   # (X86 usage)
IMAGE_DIRECTORY_ENTRY_ARCHITECTURE    =7   # Architecture Specific Data
IMAGE_DIRECTORY_ENTRY_GLOBALPTR       =8   # RVA of GP
IMAGE_DIRECTORY_ENTRY_TLS             =9   # TLS Directory
IMAGE_DIRECTORY_ENTRY_LOAD_CONFIG    =10   # Load Configuration Directory
IMAGE_DIRECTORY_ENTRY_BOUND_IMPORT   =11   # Bound Import Directory in headers
IMAGE_DIRECTORY_ENTRY_IAT            =12   # Import Address Table
IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT   =13   # Delay Load Import Descriptors
IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR =14   # COM Runtime descriptor

IMAGE_DEBUG_TYPE_UNKNOWN          =0
IMAGE_DEBUG_TYPE_COFF             =1
IMAGE_DEBUG_TYPE_CODEVIEW         =2
IMAGE_DEBUG_TYPE_FPO              =3
IMAGE_DEBUG_TYPE_MISC             =4
IMAGE_DEBUG_TYPE_EXCEPTION        =5
IMAGE_DEBUG_TYPE_FIXUP            =6
IMAGE_DEBUG_TYPE_OMAP_TO_SRC      =7
IMAGE_DEBUG_TYPE_OMAP_FROM_SRC    =8
IMAGE_DEBUG_TYPE_BORLAND          =9
IMAGE_DEBUG_TYPE_RESERVED10       =10
IMAGE_DEBUG_TYPE_CLSID            =11

SSRVOPT_CALLBACK            = 0x0001
SSRVOPT_DWORD               = 0x0002
SSRVOPT_DWORDPTR            = 0x0004
SSRVOPT_GUIDPTR             = 0x0008
SSRVOPT_OLDGUIDPTR          = 0x0010
SSRVOPT_UNATTENDED          = 0x0020
SSRVOPT_NOCOPY              = 0x0040
SSRVOPT_PARENTWIN           = 0x0080
SSRVOPT_PARAMTYPE           = 0x0100
SSRVOPT_SECURE              = 0x0200
SSRVOPT_TRACE               = 0x0400
SSRVOPT_SETCONTEXT          = 0x0800
SSRVOPT_PROXY               = 0x1000
SSRVOPT_DOWNSTREAM_STORE    = 0x2000

class IMAGE_DEBUG_DIRECTORY(Structure):
    _fields_ = [
            ("Characteristics", c_ulong),
            ("TimeDateStamp", c_ulong),
            ("MajorVersion", c_ushort),
            ("MinorVersion", c_ushort),
            ("Type", c_ulong),
            ("SizeOfData", c_ulong),
            ("AddressOfRawData", c_ulong),
            ("PointerToRawData", c_ulong),
            ]

class Win32Exception(Exception):
    def __init__(self, msg):
        Exception.__init__(self, "Win32 Error %s failed: %s" % (msg, kernel32.GetLastError()))

class Win32Mixin:
    """
    The main mixin for calling the win32 api's via ctypes
    """

    def initMixin(self):
        self.phandle = None
        self.thandles = {}
        self.win32threads = {}
        self.ImageName = ""
        self.dosdevs = []

        # We only set this when we intend to deliver it
        self.setMeta("PendingException", False)
        self.setMeta("Platform", "Win32")

        self.setupDosDeviceMaps()

        # These activities *must* all be carried out by the same
        # thread on windows.
        self.threadWrap("platformAttach", self.platformAttach)
        self.threadWrap("platformDetach", self.platformDetach)
        self.threadWrap("platformStepi", self.platformStepi)
        self.threadWrap("platformContinue", self.platformContinue)
        self.threadWrap("platformWait", self.platformWait)
        self.threadWrap("platformGetRegs", self.platformGetRegs)
        self.threadWrap("platformSetRegs", self.platformSetRegs)
        self.threadWrap("platformExec", self.platformExec)

    def setupDosDeviceMaps(self):
        self.dosdevs = []
        dname = (c_char * 512)()
        size = kernel32.GetLogicalDriveStringsA(512, addressof(dname))
        devs = dname.raw[:size-1].split("\x00")
        for dev in devs:
            dosname = "%s:" % dev[0]
            kernel32.QueryDosDeviceA("%s:" % dev[0], addressof(dname), 512)
            self.dosdevs.append( (dosname, dname.value) )

    def platformKill(self):
        kernel32.TerminateProcess(self.phandle, 0)

    def getRegisterFormat(self):
        return "7L112s16L"

    def getRegisterNames(self):
        return ("ContextFlags","debug0","debug1","debug2","debug3",
                "debug6","debug7","floatstuff","gs","fs","es","ds",
                "edi","esi","ebx","edx","ecx","eax","ebp","eip","cs","eflags",
                "esp","ss")

    def platformExec(self, cmdline):
        sinfo = STARTUPINFO()
        pinfo = PROCESS_INFORMATION()
        if not kernel32.CreateProcessA(0, cmdline, 0, 0, 0, 
                DEBUG_ONLY_THIS_PROCESS, 0, 0, addressof(sinfo), addressof(pinfo)):
            raise Exception("CreateProcess failed!")

        return pinfo.ProcessId

    def platformInjectSo(self, filename):
        try:
            lla = self.parseExpression("kernel32.LoadLibraryA")
        except:
            raise Exception("ERROR: symbol kernel32.LoadLibraryA not found!")
        regs = self.platformCall(lla, [filename,])
        if regs == None:
            raise Exception("ERROR: platformCall for LoadLibraryA Failed!")
        return regs.get("eax", 0)
        
    def platformAttach(self, pid):
        if not kernel32.DebugActiveProcess(pid):
            raise Win32Exception("DebugActiveProcess")

    def platformDetach(self):
        if not kernel32.DebugActiveProcessStop(self.pid):
            raise Win32Exception("DebugActiveProcessStop")
        kernel32.CloseHandle(self.phandle)
        self.phandle = None

    def platformReadMemory(self, address, size):
        btype = c_char * size
        buf = btype()
        ret = c_ulong(0)
        if not kernel32.ReadProcessMemory(self.phandle, address, addressof(buf), size, addressof(ret)):
            raise Win32Exception("ReadProcessMemory")
        return buf.raw

    def platformContinue(self):

        magic = DBG_CONTINUE

        if self.getMeta("PendingException"):
            magic = DBG_EXCEPTION_NOT_HANDLED

        self.setMeta("PendingException", False)
        if not kernel32.ContinueDebugEvent(self.pid, self.getMeta("StoppedThreadId"), magic):
            raise Win32Exception("ContinueDebugEvent")

    def platformStepi(self):
        self.setEflagsTf()
        self.platformContinue()

    def platformWriteMemory(self, address, buf):
        ret = c_ulong(0)
        if not kernel32.WriteProcessMemory(self.phandle, address, buf, len(buf), addressof(ret)):
            raise Win32Exception("WriteProcessMemory")
        return ret.value

    def platformGetRegs(self):
        ctx = CONTEXTx86()
        ctx.ContextFlags = (CONTEXT_FULL | CONTEXT_DEBUG_REGISTERS)
        thandle = self.thandles.get(self.getMeta("ThreadId", 0), None)
        if not thandle:
            raise Exception("Getting registers for unknown thread")
        if not kernel32.GetThreadContext(thandle, addressof(ctx)):
            raise Win32Exception("GetThreadContext")
        return string_at(addressof(ctx), sizeof(ctx))

    def platformSetRegs(self, bytes):
        buf = c_buffer(bytes)
        ctx = CONTEXTx86()
        thandle = self.thandles.get(self.getMeta("ThreadId", 0), None)
        if not thandle:
            raise Exception("Getting registers for unknown thread")
        if not kernel32.GetThreadContext(thandle, addressof(ctx)):
            raise Win32Exception("GetThreadContext")

        memmove(addressof(ctx), addressof(buf), len(bytes))
        ctx.ContextFlags = (CONTEXT_FULL | CONTEXT_DEBUG_REGISTERS)

        if not kernel32.SetThreadContext(thandle, addressof(ctx)):
            raise Win32Exception("SetThreadContext")

    def platformSendBreak(self):
        #FIXME make this support windows 2000
        if not kernel32.DebugBreakProcess(self.phandle):
            raise Win32Exception("DebugBreakProcess")

    def platformPs(self):
        ret = []
        pcount = 128 # Hardcoded limit of 128 processes... oh well..
        pids = (c_int * pcount)()
        fname = (c_char * 512)()
        needed = c_int(0)
        hmodule = c_int(0)
        if not psapi:
            raise Exception("psapi.dll not found to list processes")
        psapi.EnumProcesses(addressof(pids), 4*pcount, addressof(needed))
        for i in range(needed.value/4):
            memset(fname, 0, 512)
            phandle = kernel32.OpenProcess(PROCESS_ALL_ACCESS, 0, pids[i])
            if not phandle: # If we get 0, we failed to open it (perms)
                continue
            psapi.EnumProcessModules(phandle, addressof(hmodule), 4, addressof(needed))
            psapi.GetModuleBaseNameA(phandle, hmodule, addressof(fname), 512)
            ret.append( (pids[i], string_at(addressof(fname))) )
            kernel32.CloseHandle(phandle)
            kernel32.CloseHandle(hmodule)
        return ret

    def platformWait(self):
        event = DEBUG_EVENT()
        if not kernel32.WaitForDebugEvent(addressof(event), INFINITE):
            raise Win32Exception("WaitForDebugEvent")
        return event

    def platformProcessEvent(self, event):

        if event.ProcessId != self.pid:
            raise Exception("ERROR - Win32 Edge Condition One")

        ThreadId = event.ThreadId
        eventdict = {} # Each handler fills this in
        self.setMeta("Win32Event", eventdict)
        self.setMeta("StoppedThreadId", ThreadId)
        self.setMeta("ThreadId", ThreadId)

        if event.DebugEventCode == CREATE_PROCESS_DEBUG_EVENT:
           self.phandle = event.u.CreateProcessInfo.Process
           baseaddr = event.u.CreateProcessInfo.BaseOfImage
           self.ImageName = self.getMappedFileName(baseaddr) #FIXME vtrace wide?

           teb = event.u.CreateProcessInfo.ThreadLocalBase
           self.win32threads[ThreadId] = teb
           self.thandles[ThreadId] = event.u.CreateProcessInfo.Thread

           peb, = self.readMemoryFormat(teb + 0x30, "L")
           self.setMeta("PEB", peb)

           eventdict["ImageName"] = self.ImageName
           eventdict["StartAddress"] = event.u.CreateProcessInfo.StartAddress
           eventdict["ThreadLocalBase"] = teb

           self.fireNotifiers(vtrace.NOTIFY_ATTACH)
           self.addLibraryBase(self.ImageName, baseaddr)

        elif event.DebugEventCode == CREATE_THREAD_DEBUG_EVENT:
            self.thandles[ThreadId] = event.u.CreateThread.Thread
            teb = event.u.CreateThread.ThreadLocalBase
            startaddr = event.u.CreateThread.StartAddress
            # Setup the event dictionary for notifiers
            eventdict["ThreadLocalBase"] = teb
            eventdict["StartAddress"] = startaddr
            self.win32threads[ThreadId] = teb
            self.fireNotifiers(vtrace.NOTIFY_CREATE_THREAD)

        elif event.DebugEventCode == EXCEPTION_DEBUG_EVENT:
            excode = event.u.Exception.ExceptionRecord.ExceptionCode
            exflags = event.u.Exception.ExceptionRecord.ExceptionFlags
            exaddr = event.u.Exception.ExceptionRecord.ExceptionAddress
            exparam = event.u.Exception.ExceptionRecord.NumberParameters
            firstChance = event.u.Exception.FirstChance

            plist = []
            for i in range(exparam):
                plist.append(event.u.Exception.ExceptionRecord.ExceptionInformation[i])

            eventdict["ExceptionCode"] = excode
            eventdict["ExceptionFlags"] = exflags
            eventdict["ExceptionAddress"] = exaddr
            eventdict["NumberParameters"] = exparam
            eventdict["FirstChance"] = bool(firstChance)
            eventdict["ExceptionInformation"] = plist

            if firstChance:

                if excode == EXCEPTION_BREAKPOINT:
                    self.setMeta("PendingException", False)
                    if not self.checkBreakpoints():
                        # On first attach, all the library load
                        # events occur, then we hit a CC.  So,
                        # if we don't find a breakpoint, notify
                        # break anyay....
                        self.fireNotifiers(vtrace.NOTIFY_BREAK)

                elif excode == EXCEPTION_SINGLE_STEP:
                    self.setMeta("PendingException", False)
                    self.fireNotifiers(vtrace.NOTIFY_STEP)

                else:
                    self.setMeta("PendingException", True)
                    self.fireNotifiers(vtrace.NOTIFY_SIGNAL)

            else:
                self.setMeta("PendingException", True)
                self.fireNotifiers(vtrace.NOTIFY_SIGNAL)

        elif event.DebugEventCode == EXIT_PROCESS_DEBUG_EVENT:
            ecode = event.u.ExitProcess.ExitCode
            eventdict["ExitCode"] = ecode
            self.setMeta("ExitCode", ecode)
            self.fireNotifiers(vtrace.NOTIFY_EXIT)
            self.platformDetach()

        elif event.DebugEventCode == EXIT_THREAD_DEBUG_EVENT:
            self.win32threads.pop(ThreadId, None)
            ecode = event.u.ExitThread.ExitCode
            eventdict["ExitCode"] = ecode
            self.setMeta("ExitCode", ecode)
            self.setMeta("ExitThread", ThreadId)
            self.fireNotifiers(vtrace.NOTIFY_EXIT_THREAD)

        elif event.DebugEventCode == LOAD_DLL_DEBUG_EVENT:
            baseaddr = event.u.LoadDll.BaseOfDll
            ImageName = self.getMappedFileName(baseaddr)
            self.addLibraryBase(ImageName, baseaddr)
            self.fireNotifiers(vtrace.NOTIFY_LOAD_LIBRARY)

        elif event.DebugEventCode == UNLOAD_DLL_DEBUG_EVENT:
            eventdict["BaseOfDll"] = event.u.UnloadDll.BaseOfDll
            self.fireNotifiers(vtrace.NOTIFY_UNLOAD_LIBRARY)

        elif event.DebugEventCode == OUTPUT_DEBUG_STRING_EVENT:
            print "WIN32 Debug String"

        else:
            print "Currently unhandled event",code


    def getMappedFileName(self, address):
        if not psapi:
            raise Exception("psapi.dll not found to get map names")
        self.requireAttached()
        fname = (c_char * 512)()
        x = psapi.GetMappedFileNameA(self.phandle, address, addressof(fname), 512)
        if not x:
            return ""
        name = fname.value
        for dosname, devname in self.dosdevs:
            if name.startswith(devname):
                return name.replace(devname, dosname)
        return name

    def getExe(self):
        self.requireAttached()
        return self.ImageName

    def platformGetMaps(self):
        ret = []
        base = 0
        mbi = MEMORY_BASIC_INFORMATION32()
        while kernel32.VirtualQueryEx(self.phandle, base, addressof(mbi), sizeof(mbi)) > 0:
            if mbi.State == MEM_COMMIT:
                prot = mbi.Protect & 0xff
                if prot == PAGE_READONLY:
                    perm = vtrace.MM_READ
                elif prot == PAGE_READWRITE:
                    perm = vtrace.MM_READ | vtrace.MM_WRITE
                elif prot == PAGE_WRITECOPY:
                    perm = vtrace.MM_READ | vtrace.MM_WRITE
                elif prot == PAGE_EXECUTE:
                    perm = vtrace.MM_EXEC
                elif prot == PAGE_EXECUTE_READ:
                    perm = vtrace.MM_EXEC | vtrace.MM_READ
                elif prot == PAGE_EXECUTE_READWRITE:
                    perm = vtrace.MM_EXEC | vtrace.MM_READ | vtrace.MM_WRITE
                elif prot == PAGE_EXECUTE_WRITECOPY:
                    perm = vtrace.MM_EXEC | vtrace.MM_READ | vtrace.MM_WRITE
                else:
                    perm = 0

                base = mbi.BaseAddress
                mname = self.getMappedFileName(base)
                ret.append( (base, mbi.RegionSize, perm, mname) )

            base += mbi.RegionSize
        return ret

    def platformGetThreads(self):
        return self.win32threads

    def platformGetFds(self):
        """
        FIXME - Not implemented yet
        """
        return []

SYMCALLBACK = WINFUNCTYPE(c_int, POINTER(SYMBOL_INFO), c_ulong, c_ulong)
PDBCALLBACK = WINFUNCTYPE(c_int, c_char_p, c_void_p)

class Win32SymbolResolver(symbase.VSymbolResolver):
    def __init__(self, filename, baseaddr, handle):
        # All locals must be in constructor because of the
        # getattr over-ride..
        self.phandle = handle
        self.doff = 0
        self.file = None
        self.doshdr = None
        self.pehdr = None
        self.sections = []
        self.debugdata = []
        self.funcflags = (SYMFLAG_FUNCTION | SYMFLAG_EXPORT)
        symbase.VSymbolResolver.__init__(self, filename, baseaddr)

    def rvaToFileOffset(self, rva):
        ret = 0
        for sname,srva,svsiz,sroff,srsiz in self.sections:
            if (srva <= rva) and (srva+svsiz >= rva):
                ret = (sroff + (rva-srva))
                break
        return ret

    def symEnumCallback(self, psym, size, ctx):
        sym = psym.contents
        #print sym.Name,sym.NameLen, hex(sym.Address),sym.Flags
        if sym.Flags & self.funcflags:
            self.addSymbol(sym.Name, sym.Address, size, vtrace.SYM_FUNCTION)
        else:
            self.addSymbol(sym.Name, sym.Address, size, vtrace.SYM_MISC)
        return True

    def symFileCallback(self, filename, nothing):
        return 0

    def parseWithDbgHelp(self):
        try:

            dbghelp.SymInitialize(self.phandle, None, False)

            x = dbghelp.SymLoadModule64(self.phandle,
                        0, 
                        c_char_p(self.filename),
                        None,
                        c_ulonglong(self.loadbase),
                        None)

            # This is for debugging which pdb got loaded
            #imghlp = IMAGEHLP_MODULE64()
            #dbghelp.SymGetModuleInfo64(None, c_ulonglong(x), addressof(imghlp))

            dbghelp.SymEnumSymbols(self.phandle,
                        c_ulonglong(self.loadbase),
                        None,
                        SYMCALLBACK(self.symEnumCallback),
                        0)

            dbghelp.SymCleanup(self.phandle)

        except Exception, e:
            traceback.print_exc()
            raise

    def readPeHeader(self):
        self.doff = 0
        dosfmt = "<30HI"
        pefmt = "<I2H3I2H"

        self.doshdr = self.readPeFmt(dosfmt, 0)
        #FIXME check mz

        # Last element in the dos header is the address
        # of the pe header...
        self.file.seek(self.doshdr[-1])
        self.pehdr = self.readPeFmt(pefmt, self.doshdr[-1])

        if self.pehdr[6] == 224: # optional header length
            #peheader + sizeof(pehdr) + offset to data dictionary
            self.doff = self.doshdr[-1] + 24 + 96

    def readPeFmt(self, fmt, offset=None):
        size = struct.calcsize(fmt)
        if offset != None:
            self.file.seek(offset)
        return struct.unpack(fmt, self.file.read(size))

    def readPeSections(self):
        """
        This assumes that readPeHeader has been called
        but doesn't assume any particular file offset
        """
        self.sections = []
        self.file.seek(self.doshdr[-1] + self.pehdr[6] + 24)  # Seek to pehdr + aoutsize
        for i in range(self.pehdr[2]): # seccnt
            (name, vsize, rva,
             rsize, roffset, RelOff,
             LineOff, RelSize, LineSize,
             Chars) = self.readPeFmt("<8s6I2HI")
            sname = name.strip("\x00")
            soff = self.loadbase + rva
            self.sections.append((sname, rva, vsize, roffset, rsize))
            self.addSymbol(sname, soff, vsize, vtrace.SYM_SECTION)

    def getDataDirectory(self, which):
        """
        Return the *file offset* and size of the given data directory entry
        """
        if self.doff == 0:
            return (0,0)
        rva, size = self.readPeFmt("<II", self.doff + (8*which))
        return (self.rvaToFileOffset(rva), size)

    def readDebugDataDir(self):
        self.debugdata = []
        dbgfmt = "<2I2H4I"
        dbgsize = struct.calcsize(dbgfmt)
        offset,size = self.getDataDirectory(IMAGE_DIRECTORY_ENTRY_DEBUG)
        if not offset:
            return
        self.file.seek(offset)
        for i in range(size / dbgsize):
            dbgent = self.readPeFmt(dbgfmt)
            self.debugdata.append(dbgent)

    def initialParse(self):
        self.file = file(self.filename, "rb")
        self.readPeHeader()
        self.readPeSections()
        self.readDebugDataDir()

    def parseExports(self):
        offset,size = self.getDataDirectory(IMAGE_DIRECTORY_ENTRY_EXPORT)
        if not offset:
            return

        (Char, DateTime, Major, Minor, Name, Base, 
        fcnt,ncnt,faddr,naddr,oaddr) = self.readPeFmt("<2I2H7I", offset)
        functions = self.readPeFmt("I"*fcnt, self.rvaToFileOffset(faddr))
        names = self.readPeFmt("I"*fcnt, self.rvaToFileOffset(naddr))
        for i in range(len(names)):
            offset = self.rvaToFileOffset(names[i])
            self.file.seek(offset)
            name = self.file.read(255).split("\x00", 1)[0]
            self.addSymbol(name, self.loadbase + functions[i], 0, vtrace.SYM_FUNCTION)

    def parseBinary(self):
        self.initialParse()
        if dbghelp:
            self.parseWithDbgHelp()
        else:
            self.parseExports()

class PEMixin:
    """
    A platform mixin to parse PE binaries
    """
    def platformGetSymbolResolver(self, filename, baseaddr):
        return Win32SymbolResolver(filename, baseaddr, self.phandle)

