# Copyright (C) 2007 Invisigoth - See LICENSE file for details
from vtrace import *

class ChildCatcher(Notifier):
    """
    A child catcher for Intel/Linux.  You may use this
    object by adding it as a notifier to a trace group.
    It will add the new children to the tracegroup as
    they are created and begin them running.  I think
    it currently requires them to be in RunForever and
    Syscall modes.
    """
    def __init__(self, tracegroup):
        Notifier.__init__(self)
        self.tracegroup = tracegroup

    def notify(self, event, trace):
        if event == NOTIFY_SYSCALL:
            eax = trace.getRegisterByName("orig_eax") # WTF linux?!?!
            if eax in [2, 190, 120]: # SYS_fork and SYS_vfork SYS_clone
                if trace.getMeta("SyscallAddr", 0) == 0:
                    addr = trace.getProgramCounter()
                    trace.setMeta("SyscallAddr", addr)
                    trace.setMeta("SyscallSaved", trace.readMemory(addr, 2))
                    trace.writeMemory(addr, "\xeb\xfe")
                else:
                    pid = trace.getRegisterByName("eax")
                    addr = trace.getMeta("SyscallAddr",0)
                    saved = trace.getMeta("SyscallSaved", "")
                    trace.setMeta("SyscallAddr", 0)
                    trace.setMeta("SyscallSaved", "")

                    trace.writeMemory(addr, saved)

                    # If we got one, fix up the kid and add to the tracer
                    if pid != -1:
                        print "Caught child:",pid
                        try:
                            newtrace = self.tracegroup.addTrace(pid)
                            newtrace.writeMemory(addr, saved)
                            # Should already be non-blocking etc...
                            newtrace.run()
                        except Exception,msg:
                            print "She escaped:",msg
